@@ -90,7 +90,8 @@ bars share the default style, and in-scope bars are configured
 with alternate styling using the *-ts-* variables."
   :type '(choice (const :tag "Out of scope" out-of-scope)
 		 (const :tag "In scope" in-scope))
-  :set #'indent-bars--custom-set)
+  :set #'indent-bars--custom-set
+  :initialize #'custom-initialize-default)
 
 (defun indent-bars-ts--add-customs ()
   "Add all the relevant custom variables for the alternate ts style."
@@ -122,7 +123,8 @@ this option."
 			:key-type (symbol :tag "Language")
 			:value-type (repeat :tag "Types" (symbol :tag "Type"))))
   :group 'indent-bars-ts
-  :set #'indent-bars--custom-set)
+  :set #'indent-bars--custom-set
+  :initialize #'custom-initialize-default)
 
 (defcustom indent-bars-treesit-scope nil
   "An alist of language and treesitter node types to emphasize.
@@ -136,13 +138,15 @@ which mirror and inherit from the normal style variables."
 			:key-type (symbol :tag "Language")
 			:value-type (repeat :tag "Types" (symbol :tag "Type"))))
   :group 'indent-bars-ts
-  :set #'indent-bars--custom-set)
+  :set #'indent-bars--custom-set
+  :initialize #'custom-initialize-default)
 
 (defcustom indent-bars-treesit-scope-min-lines 3
   "Minimum number of lines a node must span to be counted as a scope."
   :type 'integer
   :group 'indent-bars-ts
-  :set #'indent-bars--custom-set)
+  :set #'indent-bars--custom-set
+  :initialize #'custom-initialize-default)
 
 (defcustom indent-bars-treesit-ignore-blank-lines-types nil
   "Do not style blank lines when the type of node at start is in this list.
@@ -155,7 +159,8 @@ is set."
   :type '(choice (const :tag "None" nil)
 		 (repeat :tag "Node types" string))
   :group 'indent-bars-ts
-  :set #'indent-bars--custom-set)
+  :set #'indent-bars--custom-set
+  :initialize #'custom-initialize-default)
 
 (defcustom indent-bars-treesit-update-delay 0.125
   "Idle time in seconds for treesitter scope updates to occur.
@@ -534,7 +539,7 @@ To be set in `indent-bars--teardown-functions'."
 
 (defun indent-bars-ts--custom-update-scope ()
   "Update the TS scope for custom setting."
-  (when indent-bars--ts-mode
+  (when (and indent-bars--ts-mode ibtcs)
     (indent-bars-ts--update-scope1 (current-buffer))))
 (add-hook 'indent-bars-custom-set #'indent-bars-ts--custom-update-scope)
 
